"""Configuration schema for django-pipeline."""
import colander


class ConfigurationSchema(colander.MappingSchema):
    """Configuration schema for django-pipeline.

    Reference is
    https://github.com/cyberdelia/django-pipeline/blob/master/pipeline/conf.py

    """
    PIPELINE_CLOSURE_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_CLOSURE_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env closure',
    )
    PIPELINE_COFFEE_SCRIPT_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_COFFEE_SCRIPT_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env coffee',
    )
    PIPELINE_COMPILERS = colander.SchemaNode(
        colander.Sequence(),
        missing=colander.drop,
        default=[],
        children=[colander.SchemaNode(colander.String())]
    )
    PIPELINE_CSS = colander.SchemaNode(
        colander.Mapping(unknown='preserve'),
        missing=colander.null,
        default={},
        children=[
            colander.SchemaNode(
                colander.Mapping(unknown='ignore'),
                missing=colander.drop,
                default={},
                children=[
                    colander.SchemaNode(
                        colander.Sequence(),
                        name='source_filenames',
                        missing=colander.required,
                        default=[],
                        children=[colander.SchemaNode(colander.String())]
                    ),
                    colander.SchemaNode(
                        colander.String(),
                        name='output_filename',
                        missing=colander.required,
                        default='',
                    ),
                    colander.SchemaNode(
                        colander.String(),
                        name='variant',
                        missing=colander.drop,
                        default=colander.null,
                    ),
                    colander.SchemaNode(
                        colander.String(),
                        name='template_name',
                        missing=colander.drop,
                        default=colander.null,
                    ),
                    colander.SchemaNode(
                        colander.Mapping(unknown='preserve'),
                        name='extra_context',
                        missing=colander.drop,
                        default={},
                    ),
                    colander.SchemaNode(
                        colander.Boolean(),
                        name='manifest',
                        missing=colander.drop,
                        default=True,
                    ),
                ]
            ),
        ]
    )
    PIPELINE_CSS_COMPRESSOR = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='pipeline.compressors.yuglify.YuglifyCompressor',
    )
    PIPELINE_CSSMIN_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_CSSMIN_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env cssmin',
    )
    PIPELINE_CSSTIDY_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='--template=highest',
    )
    PIPELINE_CSSTIDY_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env csstidy',
    )
    PIPELINE_DISABLE_WRAPPER = colander.SchemaNode(
        colander.Boolean(),
        missing=colander.drop,
        default=False,
    )
    PIPELINE_EMBED_MAX_IMAGE_SIZE = colander.SchemaNode(
        colander.Integer(),
        missing=colander.drop,
        default=32700,
    )
    PIPELINE_EMBED_PATH = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='[/]?embed/',
    )
    PIPELINE_ENABLED = colander.SchemaNode(
        colander.Boolean(),
        missing=colander.drop,
        default=colander.null,
    )
    PIPELINE_JS = colander.SchemaNode(
        colander.Mapping(unknown='preserve'),
        missing=colander.null,
        default={},
        children=[
            colander.SchemaNode(
                colander.Mapping(unknown='ignore'),
                missing=colander.drop,
                default={},
                children=[
                    colander.SchemaNode(
                        colander.Sequence(),
                        name='source_filenames',
                        missing=colander.required,
                        default=[],
                        children=[colander.SchemaNode(colander.String())]
                    ),
                    colander.SchemaNode(
                        colander.String(),
                        name='output_filename',
                        missing=colander.required,
                        default='',
                    ),
                    colander.SchemaNode(
                        colander.String(),
                        name='variant',
                        missing=colander.drop,
                        default=colander.null,
                    ),
                    colander.SchemaNode(
                        colander.String(),
                        name='template_name',
                        missing=colander.drop,
                        default=colander.null,
                    ),
                    colander.SchemaNode(
                        colander.Mapping(unknown='preserve'),
                        name='extra_context',
                        missing=colander.drop,
                        default={},
                    ),
                    colander.SchemaNode(
                        colander.Boolean(),
                        name='manifest',
                        missing=colander.drop,
                        default=True,
                    ),
                ]
            ),
        ]
    )
    PIPELINE_JS_COMPRESSOR = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='pipeline.compressors.yuglify.YuglifyCompressor',
    )
    PIPELINE_LESS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_LESS_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env lessc',
    )
    PIPELINE_LIVE_SCRIPT_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_LIVE_SCRIPT_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env lsc',
    )
    PIPELINE_MIMETYPES = colander.SchemaNode(
        colander.Sequence(),
        missing=colander.drop,
        default=(
            ('text/coffeescript', '.coffee'),
            ('text/less', '.less'),
            ('text/javascript', '.js'),
            ('text/x-sass', '.sass'),
            ('text/x-scss', '.scss'),
        ),
        children=[
            colander.SchemaNode(
                colander.Tuple(),
                missing=colander.drop,
                default=colander.null,
                children=[
                    colander.SchemaNode(colander.String()),
                    colander.SchemaNode(colander.String()),
                ]
            ),
        ]
    )
    PIPELINE_ROOT = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default=colander.null,
    )
    PIPELINE_SASS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='--update',
    )
    PIPELINE_SASS_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env sass',
    )
    PIPELINE_STORAGE = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='pipeline.storage.PipelineFinderStorage',
    )
    PIPELINE_STYLUS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_STYLUS_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env stylus',
    )
    PIPELINE_TEMPLATE_EXT = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='.jst',
    )
    PIPELINE_TEMPLATE_FUNC = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='template',
    )
    PIPELINE_TEMPLATE_NAMESPACE = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='window.JST',
    )
    PIPELINE_TEMPLATE_SEPARATOR = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='_',
    )
    PIPELINE_UGLIFYJS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_UGLIFYJS_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env uglifyjs',
    )
    PIPELINE_URL = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default=colander.null,
    )
    PIPELINE_YUGLIFY_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env yuglify',
    )
    PIPELINE_YUGLIFY_CSS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='--terminal',
    )
    PIPELINE_YUGLIFY_JS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='--terminal',
    )
    PIPELINE_YUI_BINARY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='/usr/bin/env yuicompressor',
    )
    PIPELINE_YUI_CSS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )
    PIPELINE_YUI_JS_ARGUMENTS = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='',
    )



"""Configuration schema for django-pimpmytheme."""
import colander


class ConfigurationSchema(colander.MappingSchema):
    """Configuration schema for django-pimpmytheme.

    Reference is https://pypi.python.org/pypi/django-pimpmytheme

    """
    CUSTOM_THEME_LOOKUP_OBJECT = colander.SchemaNode(
        colander.String(),
        missing='django.contrib.sites.models.Site',
        default='django.contrib.sites.models.Site',
    )
    CUSTOM_THEME_LOOKUP_ATTR = colander.SchemaNode(
        colander.String(),
        missing='name',
        default='name',
    )
    PIMPMYTHEME_FOLDER = colander.SchemaNode(
        colander.String(),
        missing=colander.required,
        default='',
    )
    PIMPMYTHEME_GIT_REPOSITORY = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default=colander.null,
    )



# -*- coding: utf-8 -*-
"""API shortcuts."""
from django_confit.loaders import load_module  # NoQA
from django_confit.loaders import load_file  # NoQA
from django_confit.loaders import load_mapping  # NoQA
from django_confit.schemas import DjangoConfigurationSchema  # NoQA
from django_confit.schemas import validate_settings  # NoQA



"""Custom colander nodes."""
from __future__ import absolute_import

import colander


class TupleOrNone(colander.Tuple):
    """A node that can be either a tuple, or ``None``."""
    def _validate(self, node, value):
        """Accept None as a valid value."""
        if value is None:
            return colander.null
        else:
            return super(TupleOrNone, self)._validate(node, value)

    def _impl(self, node, value, callback):
        if value is colander.null or value is None:
            return colander.null
        else:
            return super(TupleOrNone, self)._impl(node, value, callback)



"""Configuration schema(s) for django-nose."""
import colander


class ConfigurationSchema(colander.MappingSchema):
    NOSE_ARGS = colander.SchemaNode(
        colander.Sequence(),
        missing=[],
        default=[],
        children=[colander.SchemaNode(colander.String())]
    )
    NOSE_PLUGINS = colander.SchemaNode(
        colander.Sequence(),
        missing=[],
        default=[],
        children=[colander.SchemaNode(colander.String())]
    )



# -*- coding: utf-8 -*-
"""Utilities to load configuration from various sources:

* from :attr:`os.environ` or similar dictionary:
  :func:`settings_from_string_mapping`;

* from Python module: :func:`settings_from_module`;

* from JSON or YAML file: :func:`settings_from_file`.

"""
import json

import six
import yaml


def load_mapping(input, prefix=''):
    """Convert mapping of {key: string} to {key: complex type}.

    This function makes it possible (and easy) to load complex types from
    single-level key-value stores, such as environment variables or INI files.

    Of course, both flat and nested mappings are supported:

    >>> from django_confit import load_mapping
    >>> flat_mapping = {'DEBUG': 'True', 'SECRET_KEY': 'not a secret'}
    >>> output = load_mapping(flat_mapping)
    >>> output == flat_mapping
    True

    >>> nested_mapping = {'DATABASES': {'USER': 'me', 'HOST': 'localhost'}}
    >>> output = load_mapping(nested_mapping)
    >>> output == nested_mapping
    True

    Values can be complex types (sequences, mappings) using JSON or YAML.
    Keys using ".json" or ".yaml" suffix are automatically decoded:

    >>> nested_mapping = {
    ...     'DATABASES.yaml': 'ENGINE: sqlite3',
    ... }
    >>> output = load_mapping(nested_mapping)
    >>> output['DATABASES'] == {'ENGINE': 'sqlite3'}
    True

    You can use optional ``prefix`` argument to load only a subset of mapping:

    >>> mapping = {'YES_ONE': '1', 'NO_TWO': '2'}
    >>> load_mapping(mapping, prefix='YES_')
    {'ONE': '1'}

    """
    output = {}
    for key, value in six.iteritems(input):
        if key.startswith(prefix):
            key = key[len(prefix):]
            if key.endswith('.json'):
                output[key[:-5]] = json.loads(value)
            elif key.endswith('.yaml'):
                output[key[:-5]] = yaml.load(value)
            else:
                output[key] = value
    return output


def load_file(file_obj):
    """Return mapping from file object, using ``name`` attr to guess format.

    Supported file formats are JSON and YAML. The lowercase extension is used
    to guess the file type.

    >>> from django_confit import load_file
    >>> from six.moves import StringIO
    >>> file_obj = StringIO('SOME_LIST: [a, b, c]')
    >>> file_obj.name = 'something.yaml'
    >>> load_file(file_obj) == {
    ...     'SOME_LIST': ['a', 'b', 'c'],
    ... }
    True

    """
    file_name = file_obj.name
    if file_name.endswith('.yaml'):
        return yaml.load(file_obj)
    elif file_name.endswith('.json'):
        return json.load(file_obj)
    else:
        raise ValueError(
            'Cannot guess format of configuration file "{name}". '
            'Expected one of these extensions: "{extensions}".'.format(
                name=file_name,
                extensions='", "'.join('.yaml', '.json')))


def load_module(module_path):
    """Return module's globals as a dict.

    >>> from django_confit import load_module
    >>> settings = load_module('django.conf.global_settings')
    >>> settings['DATABASES']
    {}

    It does not load "protected" and "private" attributes (those with
    underscores).

    >>> '__name__' in settings
    False

    """
    module = __import__(module_path, fromlist='*', level=0)

    def is_uppercase(value):
        return value.upper() == value

    def is_special(value):
        return value.startswith('_')

    return dict([(key, value) for key, value in module.__dict__.items()
                 if is_uppercase(key) and not is_special(key)])



"""Unittests for the Django 1.5 branch."""
import unittest

import colander
import django

from django_confit import DjangoConfigurationSchema


@unittest.skipIf(
    django.VERSION[0] != 1 or django.VERSION[1] != 6,
    "Test for Django 1.6 only")
class Django1_6_ConfigurationSchemaTestCase(unittest.TestCase):
    def setUp(self):
        super(Django1_6_ConfigurationSchemaTestCase, self).setUp()
        #: Minimal valid Django settings.
        self.minimal_settings = {
            'DATABASES': {},
            'INSTALLED_APPS': [],
            'ROOT_URLCONF': 'fake.urls',
            'SECRET_KEY': '42',
        }

    def test_required(self):
        """Minimal settings for Django 1.6."""
        schema = DjangoConfigurationSchema()
        # Validate empty settings: invalid because some settings are required.
        raw_settings = {}
        with self.assertRaises(colander.Invalid):
            schema.deserialize(raw_settings)
        # Validate minimal settings: pass.
        cleaned_settings = schema.deserialize(self.minimal_settings)
        self.assertEqual(cleaned_settings['SECRET_KEY'], '42')
        self.assertEqual(cleaned_settings['INSTALLED_APPS'], [])
        self.assertEqual(cleaned_settings['DATABASES'], {'default': None})
        self.assertEqual(cleaned_settings['ROOT_URLCONF'], 'fake.urls')

    def test_template_loaders(self):
        """TEMPLATE_LOADERS accepts either strings or tuples."""
        schema = DjangoConfigurationSchema()
        # Just strings.
        raw_settings = self.minimal_settings
        raw_settings['TEMPLATE_LOADERS'] = [
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
        ]
        cleaned_settings = schema.deserialize(raw_settings)  # Is valid.
        self.assertEqual(cleaned_settings['TEMPLATE_LOADERS'],
                         raw_settings['TEMPLATE_LOADERS'])
        # Strings and tuples.
        raw_settings = self.minimal_settings
        raw_settings['TEMPLATE_LOADERS'] = [
            'some.string',
            (
                'django.template.loaders.cached.Loader',
                (
                    'django.template.loaders.filesystem.Loader',
                    'django.template.loaders.app_directories.Loader',
                )
            ),
        ]
        cleaned_settings = schema.deserialize(raw_settings)  # Is valid.
        self.assertEqual(cleaned_settings['TEMPLATE_LOADERS'],
                         raw_settings['TEMPLATE_LOADERS'])



# -*- coding: utf-8 -*-
"""Helpers to manage local (project-level, environment-level) settings."""
import pkg_resources


#: Module version, as defined in PEP-0396.
__version__ = pkg_resources.get_distribution(__package__.replace('-', '_')) \
                           .version


# API shortcuts.
from django_confit.api import *  # NoQA



"""Configuration schema(s) for Raven."""
import colander


class ConfigurationSchema(colander.Schema):
    """Configuration for Raven, the official Sentry client."""
    RAVEN_CONFIG = colander.SchemaNode(
        colander.Mapping(unknown='raise'),
        missing=colander.required,
        default=colander.null,
        children=[
            colander.SchemaNode(
                colander.String(),
                name='dsn',
                missing=colander.required,
                default=colander.null,
            )
        ]
    )
    SENTRY_CLIENT = colander.SchemaNode(
        colander.String(),
        missing=colander.drop,
        default='raven.contrib.django.raven_compat.DjangoClient',
    )



"""Schemas for configuration validation."""
from __future__ import absolute_import
import warnings

import django as django_lib

from django_confit.utils.importlib import import_member


def get_django_schema_class():
    """Return colander schema class for current (installed) Django version."""
    schema_import_path = \
        'django_confit.schemas.django.django_{major}_{minor}' \
        '.Django{major}_{minor}_{micro}ConfigurationSchema' \
        .format(
            major=django_lib.VERSION[0],
            minor=django_lib.VERSION[1],
            micro=django_lib.VERSION[2])
    try:
        return import_member(schema_import_path)
    except ImportError as e:
        raise ImportError(
            'Could not import "{schema_path}". '
            'Django version {version:s} is not supported by django_confit. \n'
            '{exception}'
            .format(
                schema_path=schema_import_path,
                version=django_lib.VERSION,
                exception=e,
            )
        )


DjangoConfigurationSchema = get_django_schema_class()


def composite_schema_class(installed_apps, mapping={}):
    """Return colander schema class for current Django and installed apps.

    Tries to load schemas of applications as mentioned in ``mapping`` or
    builtin django_confit:

    1. ``mapping[<APP>]``

    2. ``django_confit.schemas.<APP>.ConfigurationSchema``

    """
    bases = [DjangoConfigurationSchema]
    for app in installed_apps:
        locations = []
        try:
            locations.append(mapping[app])
        except KeyError:
            pass
        locations.append(
            'django_confit.schemas.{app}.ConfigurationSchema'.format(app=app))
        for schema_path in locations:
            try:
                bases.append(import_member(schema_path))
            except ImportError:
                pass
            else:
                break
    return type('CompositeConfigurationSchema', tuple(bases), {})


def composite_schema(installed_apps, mapping={}):
    """Return schema instance using ``installed_apps``."""
    schema_class = composite_schema_class(installed_apps, mapping)
    return schema_class()


def validate_settings(raw_settings):
    """Return cleaned settings using schemas collected from INSTALLED_APPS."""
    # Perform early validation on Django's INSTALLED_APPS.
    installed_apps = raw_settings['INSTALLED_APPS']
    schemas_mapping = raw_settings.get('CONFIT_SCHEMAS', {})
    # Create schema instance using INSTALLED_APPS.
    settings_schema = composite_schema(
        installed_apps=installed_apps,
        mapping=schemas_mapping)
    # Actually validate settings.
    cleaned_settings = settings_schema.deserialize(raw_settings)
    # Highlight differences between raw and cleaned settings.
    # Warn users when raw settings contain directives that are not used in
    # schemas.
    raw_keys = set(raw_settings.keys())
    cleaned_keys = set(cleaned_settings.keys())
    unused_keys = raw_keys.difference(cleaned_keys)
    if unused_keys:
        warnings.warn(
            'The following settings are mentioned in your configuration, but '
            'are not in cleaned settings. They may be missing in '
            'configuration schemas, or you do not need to set them up: \n'
            '- {settings}'.format(settings='\n- '.join(unused_keys)),
            Warning)
    # Return.
    return cleaned_settings



"""Tests around django-confit schemas features."""
import difflib
import unittest

import django_confit


class CompositeConfigurationSchemaTestCase(unittest.TestCase):
    def test_empty_installed_apps(self):
        """With empty INSTALLED_APPS, only Django schema is used."""
        raw_settings = {
            'DATABASES': {},
            'INSTALLED_APPS': [],
            'SECRET_KEY': '42',
        }
        cleaned_settings = django_confit.validate_settings(raw_settings)
        self.assertEqual(cleaned_settings['SECRET_KEY'], '42')
        self.assertFalse('PIPELINE_CSS' in cleaned_settings)

    def test_pipeline_installed_apps(self):
        """With 'pipeline' in INSTALLED_APPS, 'pipeline' schema is added."""
        raw_settings = {
            'DATABASES': {},
            'INSTALLED_APPS': ['pipeline'],
            'ROOT_URLCONF': 'fake.urls',
            'SECRET_KEY': '42',
        }
        cleaned_settings = django_confit.validate_settings(raw_settings)
        self.assertEqual(cleaned_settings['SECRET_KEY'], '42')
        self.assertTrue('PIPELINE_CSS' in cleaned_settings)


class DjangoConfigurationSchemaTestCase(unittest.TestCase):
    def test_default_keys(self):
        """Django settings and DjangoConfigurationSchema have the same keys."""
        django_defaults = django_confit.load_module(
            'django.conf.global_settings')
        django_keys = list(django_defaults.keys())
        django_keys.sort()
        raw_settings = {  # Minimal settings for validation.
            'DATABASES': {},
            'INSTALLED_APPS': [],
            'SECRET_KEY': '42',
        }
        confit_defaults = django_confit.validate_settings(raw_settings)
        confit_keys = list(confit_defaults.keys())
        confit_keys.sort()
        diff = difflib.unified_diff(django_keys, confit_keys)
        self.assertEqual(
            django_keys,
            confit_keys,
            '\n'.join(diff))



"""Configuration schema(s) for django-debug-toolbar."""
import colander


class ConfigurationSchema(colander.MappingSchema):
    DEBUG_TOOLBAR_CONFIG = colander.SchemaNode(
        colander.Mapping(unknown='preserve')
    )
    DEBUG_TOOLBAR_PATCH_SETTINGS = colander.SchemaNode(
        colander.Boolean(),
        missing=colander.drop,
        default=False,
    )
    DEBUG_TOOLBAR_PANELS = colander.SchemaNode(
        colander.Sequence(),
        missing=colander.drop,
        default=[
            'debug_toolbar.panels.versions.VersionsPanel',
            'debug_toolbar.panels.timer.TimerPanel',
            'debug_toolbar.panels.settings.SettingsPanel',
            'debug_toolbar.panels.headers.HeadersPanel',
            'debug_toolbar.panels.request.RequestPanel',
            'debug_toolbar.panels.sql.SQLPanel',
            'debug_toolbar.panels.staticfiles.StaticFilesPanel',
            'debug_toolbar.panels.templates.TemplatesPanel',
            'debug_toolbar.panels.cache.CachePanel',
            'debug_toolbar.panels.signals.SignalsPanel',
            'debug_toolbar.panels.logging.LoggingPanel',
            'debug_toolbar.panels.redirects.RedirectsPanel',
        ],
        children=[colander.SchemaNode(colander.String())]
    )



"""Configuration schemas for Django 1.8 branch."""
from __future__ import absolute_import

import django
from django.conf import global_settings
from django.contrib import messages

import colander

from django_confit.utils.colander import TupleOrNone


if django.VERSION[0] is 1 and django.VERSION[1] is 8:
    class Django1_8_0ConfigurationSchema(colander.MappingSchema):
        """Configuration schema for Django 1.8."""
        ABSOLUTE_URL_OVERRIDES = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=global_settings.ABSOLUTE_URL_OVERRIDES,
            default=global_settings.ABSOLUTE_URL_OVERRIDES,
        )
        ADMINS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.ADMINS,
            default=global_settings.ADMINS,
            children=[
                colander.SchemaNode(
                    colander.Tuple(),
                    children=[
                        colander.SchemaNode(colander.String()),
                        colander.SchemaNode(
                            colander.String(),
                            validator=colander.Email(),
                        ),
                    ]
                ),
            ]
        )
        ALLOWED_HOSTS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.ALLOWED_HOSTS,
            default=global_settings.ALLOWED_HOSTS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        ALLOWED_INCLUDE_ROOTS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.ALLOWED_INCLUDE_ROOTS,
            default=global_settings.ALLOWED_INCLUDE_ROOTS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        APPEND_SLASH = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.APPEND_SLASH,
            default=global_settings.APPEND_SLASH,
        )
        AUTHENTICATION_BACKENDS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.AUTHENTICATION_BACKENDS,
            default=global_settings.AUTHENTICATION_BACKENDS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        AUTH_USER_MODEL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.AUTH_USER_MODEL,
            default=global_settings.AUTH_USER_MODEL,
        )
        CACHES = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=global_settings.CACHES,
            default=global_settings.CACHES,
            children=[
                colander.SchemaNode(
                    colander.Mapping(unknown='raise'),
                    name='default',
                    missing=colander.drop,
                    children=[
                        colander.SchemaNode(
                            colander.String(),
                            name='BACKEND',
                            missing=colander.required,
                            default=colander.null,
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='KEY_FUNCTION',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='KEY_PREFIX',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='LOCATION',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.Mapping(unknown='preserve'),
                            name='OPTIONS',
                            missing=colander.drop,
                            default={},
                        ),
                        colander.SchemaNode(
                            colander.Integer(),
                            name='TIMEOUT',
                            missing=colander.drop,
                            default=300,
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='VERSION',
                            missing=colander.drop,
                            default=1,
                        ),
                    ]
                ),
            ]
        )
        CACHE_MIDDLEWARE_ALIAS = colander.SchemaNode(
            colander.String(),
            missing=global_settings.CACHE_MIDDLEWARE_ALIAS,
            default=global_settings.CACHE_MIDDLEWARE_ALIAS,
        )
        CACHE_MIDDLEWARE_ANONYMOUS_ONLY = colander.SchemaNode(
            colander.Boolean(),
            missing=colander.drop,  # Not in django.conf.global_settings.
            default=False,
        )
        CACHE_MIDDLEWARE_KEY_PREFIX = colander.SchemaNode(
            colander.String(),
            missing=global_settings.CACHE_MIDDLEWARE_KEY_PREFIX,
            default=global_settings.CACHE_MIDDLEWARE_KEY_PREFIX,
        )
        CACHE_MIDDLEWARE_SECONDS = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.CACHE_MIDDLEWARE_SECONDS,
            default=global_settings.CACHE_MIDDLEWARE_SECONDS,
        )
        CSRF_COOKIE_AGE = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.CSRF_COOKIE_AGE,
            default=global_settings.CSRF_COOKIE_AGE,
        )
        CSRF_COOKIE_DOMAIN = colander.SchemaNode(
            colander.String(),
            missing=global_settings.CSRF_COOKIE_DOMAIN,
            default=global_settings.CSRF_COOKIE_DOMAIN,
        )
        CSRF_COOKIE_HTTPONLY = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.CSRF_COOKIE_HTTPONLY,
            default=global_settings.CSRF_COOKIE_HTTPONLY,
        )
        CSRF_COOKIE_NAME = colander.SchemaNode(
            colander.String(),
            missing=global_settings.CSRF_COOKIE_NAME,
            default=global_settings.CSRF_COOKIE_NAME,
        )
        CSRF_COOKIE_PATH = colander.SchemaNode(
            colander.String(),
            missing=global_settings.CSRF_COOKIE_PATH,
            default=global_settings.CSRF_COOKIE_PATH,
        )
        CSRF_COOKIE_SECURE = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.CSRF_COOKIE_SECURE,
            default=global_settings.CSRF_COOKIE_SECURE,
        )
        CSRF_FAILURE_VIEW = colander.SchemaNode(
            colander.String(),
            missing=global_settings.CSRF_FAILURE_VIEW,
            default=global_settings.CSRF_FAILURE_VIEW,
        )
        DATABASES = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=colander.required,
            default=colander.null,
            children=[
                colander.SchemaNode(
                    colander.Mapping(unknown='raise'),
                    name='default',
                    missing=None,
                    children=[
                        # See django.db.utils.ConnectionHandler.ensure_defaults
                        colander.SchemaNode(
                            colander.String(),
                            name='ENGINE',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='HOST',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='NAME',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.Mapping(unknown='preserve'),
                            name='OPTIONS',
                            missing=colander.drop,
                            default={},
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='PASSWORD',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='PORT',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.String(),
                            name='USER',
                            missing=colander.drop,
                            default='',
                        ),
                        colander.SchemaNode(
                            colander.Boolean(),
                            name='ATOMIC_REQUESTS',
                            missing=colander.drop,
                            default=False,
                        ),
                        colander.SchemaNode(
                            colander.Boolean(),
                            name='AUTOCOMMIT',
                            missing=colander.drop,
                            default=True,
                        ),
                        colander.SchemaNode(
                            colander.Integer(),
                            name='CONN_MAX_AGE',
                            missing=colander.drop,
                            default=0,
                        ),
                        colander.SchemaNode(
                            colander.Mapping(unknown='raise'),
                            name='default',
                            missing=None,
                            children=[
                                colander.SchemaNode(
                                    colander.String(),
                                    name='CHARSET',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='COLLATION',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.Sequence(),
                                    name='DEPENDENCIES',
                                    missing=colander.drop,
                                    default=['default'],
                                    children=[
                                        colander.SchemaNode(colander.String()),
                                    ]
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='MIRROR',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='NAME',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.Boolean(),
                                    name='CREATE_DB',
                                    missing=colander.drop,
                                    default=True,
                                ),
                                colander.SchemaNode(
                                    colander.Boolean(),
                                    name='SERIALIZE',
                                    missing=colander.drop,
                                    default=True,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='USER',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.Boolean(),
                                    name='CREATE_USER',
                                    missing=colander.drop,
                                    default=True,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='PASSWORD',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='TBLSPACE',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='TBLSPACE_TMP',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='DATAFILE',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='DATAFILE_TMP',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='DATAFILE_MAXSIZE',
                                    missing=colander.drop,
                                    default=None,
                                ),
                                colander.SchemaNode(
                                    colander.String(),
                                    name='DATAFILE_TMP_MAXSIZE',
                                    missing=colander.drop,
                                    default=None,
                                ),
                            ]
                        ),
                    ]
                ),
            ]
        )
        DATABASE_ROUTERS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.DATABASE_ROUTERS,
            default=global_settings.DATABASE_ROUTERS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        DATE_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DATE_FORMAT,
            default=global_settings.DATE_FORMAT,
        )
        DATE_INPUT_FORMATS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.DATE_INPUT_FORMATS,
            default=global_settings.DATE_INPUT_FORMATS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        DATETIME_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DATETIME_FORMAT,
            default=global_settings.DATETIME_FORMAT,
        )
        DATETIME_INPUT_FORMATS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.DATETIME_INPUT_FORMATS,
            default=global_settings.DATETIME_INPUT_FORMATS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        DEBUG = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.DEBUG,
            default=global_settings.DEBUG,
        )
        DEBUG_PROPAGATE_EXCEPTIONS = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.DEBUG,
            default=global_settings.DEBUG,
        )
        DECIMAL_SEPARATOR = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DECIMAL_SEPARATOR,
            default=global_settings.DECIMAL_SEPARATOR,
        )
        DEFAULT_CHARSET = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_CHARSET,
            default=global_settings.DEFAULT_CHARSET,
        )
        DEFAULT_CONTENT_TYPE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_CONTENT_TYPE,
            default=global_settings.DEFAULT_CONTENT_TYPE,
        )
        DEFAULT_EXCEPTION_REPORTER_FILTER = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_EXCEPTION_REPORTER_FILTER,
            default=global_settings.DEFAULT_EXCEPTION_REPORTER_FILTER,
        )
        DEFAULT_FILE_STORAGE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_FILE_STORAGE,
            default=global_settings.DEFAULT_FILE_STORAGE,
        )
        DEFAULT_FROM_EMAIL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_FROM_EMAIL,
            default=global_settings.DEFAULT_FROM_EMAIL,
            validator=colander.Email(),
        )
        DEFAULT_INDEX_TABLESPACE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_INDEX_TABLESPACE,
            default=global_settings.DEFAULT_INDEX_TABLESPACE,
        )
        DEFAULT_TABLESPACE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.DEFAULT_TABLESPACE,
            default=global_settings.DEFAULT_TABLESPACE,
        )
        DISALLOWED_USER_AGENTS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.DISALLOWED_USER_AGENTS,
            default=global_settings.DISALLOWED_USER_AGENTS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        EMAIL_BACKEND = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_BACKEND,
            default=global_settings.EMAIL_BACKEND,
        )
        EMAIL_FILE_PATH = colander.SchemaNode(
            colander.String(),
            missing=colander.drop,  # Not in djanco.conf.globalsettings
            default=colander.null,
        )
        EMAIL_HOST = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_HOST,
            default=global_settings.EMAIL_HOST,
        )
        EMAIL_HOST_PASSWORD = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_HOST_PASSWORD,
            default=global_settings.EMAIL_HOST_PASSWORD,
        )
        EMAIL_HOST_USER = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_HOST_USER,
            default=global_settings.EMAIL_HOST_USER,
        )
        EMAIL_PORT = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.EMAIL_PORT,
            default=global_settings.EMAIL_PORT,
        )
        EMAIL_SUBJECT_PREFIX = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_SUBJECT_PREFIX,
            default=global_settings.EMAIL_SUBJECT_PREFIX,
        )
        EMAIL_SSL_CERTFILE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_SSL_CERTFILE,
            default=global_settings.EMAIL_SSL_CERTFILE,
        )
        EMAIL_SSL_KEYFILE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.EMAIL_SSL_KEYFILE,
            default=global_settings.EMAIL_SSL_KEYFILE,
        )
        EMAIL_TIMEOUT = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.EMAIL_TIMEOUT,
            default=global_settings.EMAIL_TIMEOUT,
        )
        EMAIL_USE_TLS = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.EMAIL_USE_TLS,
            default=global_settings.EMAIL_USE_TLS,
        )
        EMAIL_USE_SSL = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.EMAIL_USE_SSL,
            default=global_settings.EMAIL_USE_SSL,
        )
        FORMAT_MODULE_PATH = colander.SchemaNode(
            colander.String(),
            missing=global_settings.FORMAT_MODULE_PATH,
            default=global_settings.FORMAT_MODULE_PATH,
        )
        FILE_CHARSET = colander.SchemaNode(
            colander.String(),
            missing=global_settings.FILE_CHARSET,
            default=global_settings.FILE_CHARSET,
        )
        FILE_UPLOAD_DIRECTORY_PERMISSIONS = colander.SchemaNode(
            colander.String(),
            missing=global_settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS,
            default=global_settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS,
        )
        FILE_UPLOAD_HANDLERS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.FILE_UPLOAD_HANDLERS,
            default=global_settings.FILE_UPLOAD_HANDLERS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        FILE_UPLOAD_MAX_MEMORY_SIZE = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.FILE_UPLOAD_MAX_MEMORY_SIZE,
            default=global_settings.FILE_UPLOAD_MAX_MEMORY_SIZE,
        )
        FILE_UPLOAD_PERMISSIONS = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.FILE_UPLOAD_PERMISSIONS,
            default=global_settings.FILE_UPLOAD_PERMISSIONS,
        )
        FILE_UPLOAD_TEMP_DIR = colander.SchemaNode(
            colander.String(),
            missing=global_settings.FILE_UPLOAD_TEMP_DIR,
            default=global_settings.FILE_UPLOAD_TEMP_DIR,
        )
        FIRST_DAY_OF_WEEK = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.FIRST_DAY_OF_WEEK,
            default=global_settings.FIRST_DAY_OF_WEEK,
        )
        FIXTURE_DIRS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.FIXTURE_DIRS,
            default=global_settings.FIXTURE_DIRS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        FORCE_SCRIPT_NAME = colander.SchemaNode(
            colander.String(),
            missing=global_settings.FORCE_SCRIPT_NAME,
            default=global_settings.FORCE_SCRIPT_NAME,
        )
        IGNORABLE_404_URLS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.IGNORABLE_404_URLS,
            default=global_settings.IGNORABLE_404_URLS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        INSTALLED_APPS = colander.SchemaNode(
            colander.Sequence(),
            missing=colander.required,
            default=colander.null,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        INTERNAL_IPS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.INTERNAL_IPS,
            default=global_settings.INTERNAL_IPS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        LANGUAGE_CODE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LANGUAGE_CODE,
            default=global_settings.LANGUAGE_CODE,
        )
        LANGUAGE_COOKIE_AGE = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.LANGUAGE_COOKIE_AGE,
            default=global_settings.LANGUAGE_COOKIE_AGE,
        )
        LANGUAGE_COOKIE_DOMAIN = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LANGUAGE_COOKIE_DOMAIN,
            default=global_settings.LANGUAGE_COOKIE_DOMAIN,
        )
        LANGUAGE_COOKIE_NAME = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LANGUAGE_COOKIE_NAME,
            default=global_settings.LANGUAGE_COOKIE_NAME,
        )
        LANGUAGE_COOKIE_PATH = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LANGUAGE_COOKIE_PATH,
            default=global_settings.LANGUAGE_COOKIE_PATH,
        )
        LANGUAGES = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.LANGUAGES,
            default=global_settings.LANGUAGES,
            children=[
                colander.SchemaNode(
                    colander.Tuple(),
                    children=[
                        colander.SchemaNode(colander.String()),
                        colander.SchemaNode(colander.String()),
                    ]
                ),
            ]
        )
        LANGUAGES_BIDI = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.LANGUAGES_BIDI,
            default=global_settings.LANGUAGES_BIDI,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        LOCALE_PATHS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.LOCALE_PATHS,
            default=global_settings.LOCALE_PATHS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        LOGGING = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=global_settings.LOGGING,
            default=global_settings.LOGGING,
        )
        LOGGING_CONFIG = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LOGGING_CONFIG,
            default=global_settings.LOGGING_CONFIG,
        )
        LOGIN_REDIRECT_URL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LOGIN_REDIRECT_URL,
            default=global_settings.LOGIN_REDIRECT_URL,
        )
        LOGIN_URL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LOGIN_URL,
            default=global_settings.LOGIN_URL,
        )
        LOGOUT_URL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.LOGOUT_URL,
            default=global_settings.LOGOUT_URL,
        )
        MANAGERS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.MANAGERS,
            default=global_settings.MANAGERS,
            children=[
                colander.SchemaNode(
                    colander.Tuple(),
                    children=[
                        colander.SchemaNode(colander.String()),
                        colander.SchemaNode(
                            colander.String(),
                            validator=colander.Email(),
                        ),
                    ]
                ),
            ]
        )
        MEDIA_ROOT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.MEDIA_ROOT,
            default=global_settings.MEDIA_ROOT,
        )
        MEDIA_URL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.MEDIA_URL,
            default=global_settings.MEDIA_URL,
        )
        MESSAGE_LEVEL = colander.SchemaNode(
            colander.Integer(),
            missing=colander.drop,  # Not in django.conf.global_settings
            default=messages.INFO,
        )
        MESSAGE_STORAGE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.MESSAGE_STORAGE,
            default=global_settings.MESSAGE_STORAGE,
        )
        MESSAGE_TAGS = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=colander.drop,  # Not in django.conf.global_settings
            default=messages.DEFAULT_TAGS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        MIDDLEWARE_CLASSES = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.MIDDLEWARE_CLASSES,
            default=global_settings.MIDDLEWARE_CLASSES,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        MIGRATION_MODULES = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=global_settings.MIGRATION_MODULES,
            default=global_settings.MIGRATION_MODULES,
        )
        MONTH_DAY_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.MONTH_DAY_FORMAT,
            default=global_settings.MONTH_DAY_FORMAT,
        )
        NUMBER_GROUPING = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.NUMBER_GROUPING,
            default=global_settings.NUMBER_GROUPING,
        )
        PASSWORD_HASHERS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.PASSWORD_HASHERS,
            default=global_settings.PASSWORD_HASHERS,
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        PASSWORD_RESET_TIMEOUT_DAYS = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.PASSWORD_RESET_TIMEOUT_DAYS,
            default=global_settings.PASSWORD_RESET_TIMEOUT_DAYS,
        )
        PREPEND_WWW = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.PREPEND_WWW,
            default=global_settings.PREPEND_WWW,
        )
        RESTRUCTUREDTEXT_FILTER_SETTINGS = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=colander.drop,  # Not in django.conf.global_settings
            default={},
        )
        ROOT_URLCONF = colander.SchemaNode(
            colander.String(),
            missing=colander.drop,  # Not in django.conf.global_settings
            default=colander.null,
        )
        SECRET_KEY = colander.SchemaNode(
            colander.String(),
            missing=colander.required,
            default=colander.null,
        )
        SECURE_BROWSER_XSS_FILTER = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SECURE_BROWSER_XSS_FILTER,
            default=global_settings.SECURE_BROWSER_XSS_FILTER,
        )
        SECURE_CONTENT_TYPE_NOSNIFF = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SECURE_CONTENT_TYPE_NOSNIFF,
            default=global_settings.SECURE_CONTENT_TYPE_NOSNIFF,
        )
        SECURE_HSTS_INCLUDE_SUBDOMAINS = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SECURE_HSTS_INCLUDE_SUBDOMAINS,
            default=global_settings.SECURE_HSTS_INCLUDE_SUBDOMAINS,
        )
        SECURE_HSTS_SECONDS = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.SECURE_HSTS_SECONDS,
            default=global_settings.SECURE_HSTS_SECONDS,
        )
        SECURE_REDIRECT_EXEMPT = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.SECURE_REDIRECT_EXEMPT,
            default=global_settings.SECURE_REDIRECT_EXEMPT,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        SECURE_PROXY_SSL_HEADER = colander.SchemaNode(
            TupleOrNone(),
            missing=global_settings.SECURE_PROXY_SSL_HEADER,
            default=global_settings.SECURE_PROXY_SSL_HEADER,
            children=[
                colander.SchemaNode(colander.String()),
                colander.SchemaNode(colander.String()),
            ]
        )
        SECURE_SSL_HOST = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SECURE_SSL_HOST,
            default=global_settings.SECURE_SSL_HOST,
        )
        SECURE_SSL_REDIRECT = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SECURE_SSL_REDIRECT,
            default=global_settings.SECURE_SSL_REDIRECT,
        )
        SERIALIZATION_MODULES = colander.SchemaNode(
            colander.Mapping(unknown='preserve'),
            missing=colander.drop,
            default={},
            children=[
                colander.SchemaNode(
                    colander.String(),
                ),
            ]
        )
        SERVER_EMAIL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SERVER_EMAIL,
            default=global_settings.SERVER_EMAIL,
        )
        SESSION_COOKIE_AGE = colander.SchemaNode(
            colander.Integer(),
            missing=global_settings.SESSION_COOKIE_AGE,
            default=global_settings.SESSION_COOKIE_AGE,
        )
        SESSION_COOKIE_DOMAIN = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_COOKIE_DOMAIN,
            default=global_settings.SESSION_COOKIE_DOMAIN,
        )
        SESSION_COOKIE_HTTPONLY = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SESSION_COOKIE_HTTPONLY,
            default=global_settings.SESSION_COOKIE_HTTPONLY,
        )
        SESSION_COOKIE_NAME = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_COOKIE_NAME,
            default=global_settings.SESSION_COOKIE_NAME,
        )
        SESSION_COOKIE_PATH = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_COOKIE_PATH,
            default=global_settings.SESSION_COOKIE_PATH,
        )
        SESSION_CACHE_ALIAS = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_CACHE_ALIAS,
            default=global_settings.SESSION_CACHE_ALIAS,
        )
        SESSION_COOKIE_SECURE = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SESSION_COOKIE_SECURE,
            default=global_settings.SESSION_COOKIE_SECURE,
        )
        SESSION_ENGINE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_ENGINE,
            default=global_settings.SESSION_ENGINE,
        )
        SESSION_EXPIRE_AT_BROWSER_CLOSE = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SESSION_EXPIRE_AT_BROWSER_CLOSE,
            default=global_settings.SESSION_EXPIRE_AT_BROWSER_CLOSE,
        )
        SESSION_FILE_PATH = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_FILE_PATH,
            default=global_settings.SESSION_FILE_PATH,
        )
        SESSION_SAVE_EVERY_REQUEST = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.SESSION_SAVE_EVERY_REQUEST,
            default=global_settings.SESSION_SAVE_EVERY_REQUEST,
        )
        SESSION_SERIALIZER = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SESSION_SERIALIZER,
            default=global_settings.SESSION_SERIALIZER,
        )
        SHORT_DATE_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SHORT_DATE_FORMAT,
            default=global_settings.SHORT_DATE_FORMAT,
        )
        SHORT_DATETIME_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SHORT_DATETIME_FORMAT,
            default=global_settings.SHORT_DATETIME_FORMAT,
        )
        SIGNING_BACKEND = colander.SchemaNode(
            colander.String(),
            missing=global_settings.SIGNING_BACKEND,
            default=global_settings.SIGNING_BACKEND,
        )
        SILENCED_SYSTEM_CHECKS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.SILENCED_SYSTEM_CHECKS,
            default=global_settings.SILENCED_SYSTEM_CHECKS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        SITE_ID = colander.SchemaNode(
            colander.Integer(),
            missing=colander.drop,
            default=colander.null,
        )
        STATIC_ROOT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.STATIC_ROOT,
            default=global_settings.STATIC_ROOT,
        )
        STATIC_URL = colander.SchemaNode(
            colander.String(),
            missing=global_settings.STATIC_URL,
            default=global_settings.STATIC_URL,
        )
        STATICFILES_DIRS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.STATICFILES_DIRS,
            default=global_settings.STATICFILES_DIRS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        STATICFILES_FINDERS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.STATICFILES_FINDERS,
            default=global_settings.STATICFILES_FINDERS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        STATICFILES_STORAGE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.STATICFILES_STORAGE,
            default=global_settings.STATICFILES_STORAGE,
        )
        TEMPLATE_CONTEXT_PROCESSORS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.TEMPLATE_CONTEXT_PROCESSORS,
            default=global_settings.TEMPLATE_CONTEXT_PROCESSORS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        TEMPLATE_DEBUG = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.TEMPLATE_DEBUG,
            default=global_settings.TEMPLATE_DEBUG,
        )
        TEMPLATE_DIRS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.TEMPLATE_DIRS,
            default=global_settings.TEMPLATE_DIRS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        TEMPLATE_LOADERS = colander.SchemaNode(
            colander.List(),
            missing=global_settings.TEMPLATE_LOADERS,
            default=global_settings.TEMPLATE_LOADERS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        TEMPLATE_STRING_IF_INVALID = colander.SchemaNode(
            colander.String(),
            missing=global_settings.TEMPLATE_STRING_IF_INVALID,
            default=global_settings.TEMPLATE_STRING_IF_INVALID,
        )
        TEMPLATES = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.TEMPLATES,
            default=global_settings.TEMPLATES,
            children=[]
        )
        TEST_NON_SERIALIZED_APPS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.TEST_NON_SERIALIZED_APPS,
            default=global_settings.TEST_NON_SERIALIZED_APPS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        TEST_RUNNER = colander.SchemaNode(
            colander.String(),
            missing=global_settings.TEST_RUNNER,
            default=global_settings.TEST_RUNNER,
        )
        THOUSAND_SEPARATOR = colander.SchemaNode(
            colander.String(),
            missing=global_settings.THOUSAND_SEPARATOR,
            default=global_settings.THOUSAND_SEPARATOR,
        )
        TIME_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.TIME_FORMAT,
            default=global_settings.TIME_FORMAT,
        )
        TIME_INPUT_FORMATS = colander.SchemaNode(
            colander.Sequence(),
            missing=global_settings.TIME_INPUT_FORMATS,
            default=global_settings.TIME_INPUT_FORMATS,
            children=[
                colander.SchemaNode(colander.String()),
            ]
        )
        TIME_ZONE = colander.SchemaNode(
            colander.String(),
            missing=global_settings.TIME_ZONE,
            default=global_settings.TIME_ZONE,
        )
        USE_ETAGS = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.USE_ETAGS,
            default=global_settings.USE_ETAGS,
        )
        USE_I18N = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.USE_I18N,
            default=global_settings.USE_I18N,
        )
        USE_L10N = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.USE_L10N,
            default=global_settings.USE_L10N,
        )
        USE_THOUSAND_SEPARATOR = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.USE_THOUSAND_SEPARATOR,
            default=global_settings.USE_THOUSAND_SEPARATOR,
        )
        USE_TZ = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.USE_TZ,
            default=global_settings.USE_TZ,
        )
        USE_X_FORWARDED_HOST = colander.SchemaNode(
            colander.Boolean(),
            missing=global_settings.USE_X_FORWARDED_HOST,
            default=global_settings.USE_X_FORWARDED_HOST,
        )
        WSGI_APPLICATION = colander.SchemaNode(
            colander.String(),
            missing=global_settings.WSGI_APPLICATION,
            default=global_settings.WSGI_APPLICATION,
        )
        YEAR_MONTH_FORMAT = colander.SchemaNode(
            colander.String(),
            missing=global_settings.YEAR_MONTH_FORMAT,
            default=global_settings.YEAR_MONTH_FORMAT,
        )
        X_FRAME_OPTIONS = colander.SchemaNode(
            colander.String(),
            missing=global_settings.X_FRAME_OPTIONS,
            default=global_settings.X_FRAME_OPTIONS,
        )

    class Django1_8_1ConfigurationSchema(Django1_8_0ConfigurationSchema):
        """Configuration schema for Django 1.8.1."""

    class Django1_8_2ConfigurationSchema(Django1_8_1ConfigurationSchema):
        """Configuration schema for Django 1.8.2."""

    class Django1_8_3ConfigurationSchema(Django1_8_2ConfigurationSchema):
        """Configuration schema for Django 1.8.3."""

    class Django1_8_4ConfigurationSchema(Django1_8_3ConfigurationSchema):
        """Configuration schema for Django 1.8.4."""

    class Django1_8_5ConfigurationSchema(Django1_8_4ConfigurationSchema):
        """Configuration schema for Django 1.8.5."""

    class Django1_8_6ConfigurationSchema(Django1_8_5ConfigurationSchema):
        """Configuration schema for Django 1.8.6."""

    class Django1_8_7ConfigurationSchema(Django1_8_6ConfigurationSchema):
        """Configuration schema for Django 1.8.7."""

    class Django1_8_8ConfigurationSchema(Django1_8_7ConfigurationSchema):
        """Configuration schema for Django 1.8.8."""

    class Django1_8_9ConfigurationSchema(Django1_8_8ConfigurationSchema):
        """Configuration schema for Django 1.8.9."""

    class Django1_8_10ConfigurationSchema(Django1_8_9ConfigurationSchema):
        """Configuration schema for Django 1.8.10."""

    class Django1_8_11ConfigurationSchema(Django1_8_10ConfigurationSchema):
        """Configuration schema for Django 1.8.11."""

    class Django1_8_12ConfigurationSchema(Django1_8_11ConfigurationSchema):
        """Configuration schema for Django 1.8.12."""

    class Django1_8_13ConfigurationSchema(Django1_8_12ConfigurationSchema):
        """Configuration schema for Django 1.8.13."""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Python packaging."""
import os
import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    """Test command that runs tox."""
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox  # import here, cause outside the eggs aren't loaded.
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)


#: Absolute path to directory containing setup.py file.
here = os.path.abspath(os.path.dirname(__file__))
#: Boolean, ``True`` if environment is running Python version 2.
IS_PYTHON2 = sys.version_info[0] == 2


NAME = 'django-confit'
README = open(os.path.join(here, 'README.rst')).read()
DESCRIPTION = 'Django settings loaders and validators, with local flavour.'
VERSION = open(os.path.join(here, 'VERSION')).read().strip()
AUTHOR = u'Benoît Bryon'
EMAIL = 'benoit@marmelune.net'
URL = 'https://django-confit.readthedocs.org'
CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
    "License :: OSI Approved :: BSD License",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.3",
    "Framework :: Django",
]
KEYWORDS = [
    'configuration',
    'settings',
    'schema',
    'validation',
    'yaml',
    'json',
]
PACKAGES = [NAME.replace('-', '_')]
REQUIREMENTS = [
    'colander',
    'Django>=1.5',
    'PyYAML',
    'setuptools',
    'six',
]
if IS_PYTHON2:
    REQUIREMENTS.extend(['mock'])
ENTRY_POINTS = {}
LICENSE = 'BSD'
TEST_REQUIREMENTS = ['tox']
CMDCLASS = {'test': Tox}


if __name__ == '__main__':  # Do not run setup() when we import this module.
    setup(
        name=NAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=README,
        classifiers=CLASSIFIERS,
        keywords=' '.join(KEYWORDS),
        author=AUTHOR,
        author_email=EMAIL,
        url=URL,
        license=LICENSE,
        packages=PACKAGES,
        include_package_data=True,
        zip_safe=False,
        install_requires=REQUIREMENTS,
        entry_points=ENTRY_POINTS,
        tests_require=TEST_REQUIREMENTS,
        cmdclass=CMDCLASS,
    )



"""Import utilities."""
from __future__ import absolute_import


def import_member(import_string):
    """Import one member of Python module by path.

    >>> import os.path
    >>> imported = import_member('os.path.supports_unicode_filenames')
    >>> os.path.supports_unicode_filenames is imported
    True

    """
    module_name, factory_name = str(import_string).rsplit('.', 1)
    module = __import__(module_name, globals(), locals(), [factory_name], 0)
    return getattr(module, factory_name)



"""Unittests for the Django 1.5 branch."""
import unittest

import colander
import django

from django_confit import DjangoConfigurationSchema


@unittest.skipIf(
    django.VERSION[0] != 1 or django.VERSION[1] != 5,
    "Test for Django 1.5 only")
class Django1_5_ConfigurationSchemaTestCase(unittest.TestCase):
    def test_required(self):
        """Minimal settings for Django 1.5."""
        schema = DjangoConfigurationSchema()
        # Validate empty settings: invalid because some settings are required.
        raw_settings = {}
        with self.assertRaises(colander.Invalid):
            schema.deserialize(raw_settings)
        # Validate minimal settings: pass.
        raw_settings = {
            'DATABASES': {},
            'INSTALLED_APPS': [],
            'ROOT_URLCONF': 'fake.urls',
            'SECRET_KEY': '42',
        }
        cleaned_settings = schema.deserialize(raw_settings)
        self.assertEqual(cleaned_settings['SECRET_KEY'], '42')
